# Copyright (c) Stanford University, The Regents of the University of
#               California, and others.
#
# All Rights Reserved.
#
# See Copyright-SimVascular.txt for additional details.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject
# to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__),"src"))
import argparse
import numpy as np
import glob
import meshing
import models
import io_utils
import time

def create_volume_mesh(poly_fn, edge_size, output_dir):
    lvmodel = models.LeftVentricle(io_utils.read_vtk_mesh(poly_fn))
    output_vol = os.path.join(output_dir, 'mesh-complete')
    lvmodel.remesh(edge_size, poly_fn, poly_fn=None, ug_fn=output_vol, mmg=False)
    lvmodel.write_mesh_complete(output_vol)

if __name__ == '__main__':
    from utils import natural_sort
    start = time.time()
    parser = argparse.ArgumentParser()
    
    parser.add_argument('--input_dir', help="Path to the surface meshes")
    parser.add_argument('--output_dir', help="Path to the volume meshes")
    parser.add_argument('--model_out', help="Name format of surface")
    parser.add_argument('--edge_size', type=float, help="Maximum edge size of the volumetric mesh.")
    parser.add_argument('--phase', default=-1, type=int, help="Id of the phase to generate volume mesh")
    args = parser.parse_args()
    
    input_dir = args.input_dir

    if args.phase == -1:
        try:
            volume_fn = np.load(os.path.join(input_dir, "volume.npy"))
            phase = volume_fn[:,0][int(np.argmax(volume_fn[:,1]))]
        except:
            print("Mesh volumes not found, the first model will be meshed")
            phase = 0
    else:
        phase = args.phase
    surface_fns = natural_sort(glob.glob(os.path.join(args.input_dir, '*.vtp')))
    poly_fn = surface_fns[int(phase)]
    
    create_volume_mesh(poly_fn, args.edge_size, args.output_dir)
    end = time.time()
    print("Time spent in volume_mesh_main.py: ", end-start)
    print("Mesh generated for ", poly_fn)
