# Copyright (c) Stanford University, The Regents of the University of
#               California, and others.
#
# All Rights Reserved.
#
# See Copyright-SimVascular.txt for additional details.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject
# to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sv_vis as vis
import vtk
# -------------------
# ::vis::deleteWindow
# -------------------
def deleteWindow(window, ren):
    
  #@c Delete window displaying single renderer
  #@a window: window to be destroyed
  #@a ren: renderer to be destroyed

  # clear all displayed objects
  vis.vis_imgRm(ren)
  vis.polyRmAll(ren)
  vis.actorRmAll(ren)

  # delete generic interactors
  try:
      delattr(vis, "genericInteractor_"+ren[0])
      delattr(vis, "genericInteractorStyle_"+ren[0])
  except:
      print("No genericInteractor or genericInteractorStyle found")
      
  del window
  del ren


# ---------------
# ::vis::initTKgr
# ---------------
#Initialize the vtk render window
def initRen(title):
    renId = 1
    renWin = [None]*2
    renWin[0] = title
    renWin[1] = vtk.vtkRenderWindow()
    renWin[1].SetWindowName(title)
    
    ren = [None]*2
    ren[0] = "%s_ren%d" % (renWin[0], renId)
    ren[1] = vtk.vtkRenderer()
    renWin[1].AddRenderer(ren[1])
    ren[1].SetBackground(0,0,0)
    
    
    # add generic interactors
    try:
      delattr(vis, "genericInteractor_"+ren[0])
      delattr(vis, "genericInteractorStyle_"+ren[0])
    except:
      pass
      
    genericInteractor = [None]*2
    genericInteractorStyle = [None]*2
    
    genericInteractor[0] =  "iren_"+ren[0]
    genericInteractorStyle[0] = "irenStyle_"+ren[0]
    
    genericInteractor[1] = vtk.vtkRenderWindowInteractor()
    genericInteractorStyle[1] = vtk.vtkInteractorStyleSwitch()
    
    genericInteractorStyle[1].SetCurrentStyleToTrackballCamera()
    genericInteractor[1].SetInteractorStyle(genericInteractorStyle[1])
    
    genericInteractor[1].SetRenderWindow(renWin[1])
    
    setattr(vis, ren[0],ren)
    setattr(vis, renWin[0],renWin)
    setattr(vis, genericInteractor[0], genericInteractor)
    setattr(vis, genericInteractorStyle[0], genericInteractorStyle)

    return ren, renWin

    
    


#proc ::vis::deletePickedActor {ren} {
#
#  #@author Nathan Wilson
#  #@c Deletes the currently selected actor from the renderer
#  #@a ren: renderer
#
#  ::vis::tkDeselectPickedActor
#  ::vis::actorRm $::vis::tkvars(PickedAssemblyRenderer) $::vis::tkvars(PickedAssembly)
#
#}